@@ -15,11 +15,9 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Iterator
-from typing import Optional
 from typing import overload
-from typing import Union
 
+from collections.abc import Iterator
 from datetime import datetime
 from datetime import timezone
 
@@ -37,6 +35,7 @@ from nbxmpp.structs import TuneData
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.client_modules import ClientModules
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import SimpleClientState
 from gajim.common.helpers import chatstate_to_string
@@ -44,6 +43,10 @@ from gajim.common.helpers import get_groupchat_name
 from gajim.common.helpers import Observable
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.util import LogAdapter
+from gajim.common.setting_values import AllContactSettings
+from gajim.common.setting_values import AllContactSettingsT
+from gajim.common.setting_values import AllGroupChatSettings
+from gajim.common.setting_values import AllGroupChatSettingsT
 from gajim.common.setting_values import BoolContactSettings
 from gajim.common.setting_values import BoolGroupChatSettings
 from gajim.common.setting_values import IntGroupChatSettings
@@ -65,14 +68,14 @@ class ContactSettings:
     @overload
     def get(self, setting: BoolContactSettings) -> bool: ...  # noqa: E704
 
-    def get(self, setting: Any) -> Any:
+    def get(self, setting: AllContactSettings) -> AllContactSettingsT:
         return app.settings.get_contact_setting(
             self._account, self._jid, setting)
 
     @overload
-    def set(self, setting: StringContactSettings, value: str) -> None: ...  # noqa: E501, E704
+    def set(self, setting: StringContactSettings, value: str | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: BoolContactSettings, value: bool) -> None: ...  # noqa: E501, E704
+    def set(self, setting: BoolContactSettings, value: bool | None) -> None: ...  # noqa: E501, E704
 
     def set(self, setting: Any, value: Any) -> None:
         app.settings.set_contact_setting(
@@ -91,16 +94,16 @@ class GroupChatSettings:
     @overload
     def get(self, setting: IntGroupChatSettings) -> int: ...  # noqa: E704
 
-    def get(self, setting: Any) -> Any:
+    def get(self, setting: AllGroupChatSettings) -> AllGroupChatSettingsT:
         return app.settings.get_group_chat_setting(
             self._account, self._jid, setting)
 
     @overload
-    def set(self, setting: StringGroupChatSettings, value: str) -> None: ...  # noqa: E501, E704
+    def set(self, setting: StringGroupChatSettings, value: str | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: BoolGroupChatSettings, value: bool) -> None: ...  # noqa: E501, E704
+    def set(self, setting: BoolGroupChatSettings, value: bool | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: IntGroupChatSettings, value: int) -> None: ...  # noqa: E501, E704
+    def set(self, setting: IntGroupChatSettings, value: int | None) -> None: ...  # noqa: E501, E704
 
     def set(self, setting: Any, value: Any) -> None:
         app.settings.set_group_chat_setting(
@@ -111,7 +114,7 @@ class Contacts(BaseModule):
     def __init__(self, client: types.Client) -> None:
         BaseModule.__init__(self, client)
 
-        self._contacts: dict[JID, Union[BareContact, GroupchatContact]] = {}
+        self._contacts: dict[JID, BareContact | GroupchatContact] = {}
         self._con.connect_signal('state-changed', self._on_client_state_changed)
         self._con.connect_signal('resume-failed', self._on_client_resume_failed)
 
@@ -127,7 +130,7 @@ class Contacts(BaseModule):
         if state.is_disconnected:
             self._reset_presence()
 
-    def add_chat_contact(self, jid: Union[str, JID]) -> BareContact:
+    def add_chat_contact(self, jid: str | JID) -> BareContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
@@ -143,7 +146,7 @@ class Contacts(BaseModule):
         self._contacts[jid] = contact
         return contact
 
-    def add_group_chat_contact(self, jid: Union[str, JID]) -> GroupchatContact:
+    def add_group_chat_contact(self, jid: str | JID) -> GroupchatContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
@@ -160,14 +163,14 @@ class Contacts(BaseModule):
         self._contacts[jid] = contact
         return contact
 
-    def add_private_contact(self, jid: Union[str, JID]) -> GroupchatParticipant:
+    def add_private_contact(self, jid: str | JID) -> GroupchatParticipant:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
         if jid.resource is None:
             raise ValueError(f'Trying to add a bare JID as private {jid}')
 
-        contact = self._contacts.get(jid.bare)
+        contact = self._contacts.get(jid.new_as_bare())
         if contact is None:
             group_chat_contact = self.add_group_chat_contact(jid.bare)
             return group_chat_contact.add_resource(jid.resource)
@@ -179,9 +182,8 @@ class Contacts(BaseModule):
         return contact.add_resource(jid.resource)
 
     def add_contact(self,
-                    jid: Union[str, JID],
-                    groupchat: bool = False) -> Union[BareContact,
-                                                      GroupchatContact]:
+                    jid: str | JID,
+                    groupchat: bool = False) -> BareContact | GroupchatContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
@@ -198,7 +200,7 @@ class Contacts(BaseModule):
         return contact
 
     def get_contact(self,
-                    jid: Union[str, JID],
+                    jid: str | JID,
                     groupchat: bool = False
                     ) -> types.ContactT:
 
@@ -218,8 +220,9 @@ class Contacts(BaseModule):
         contact = contact.get_resource(resource)
         return contact
 
-    def get_bare_contact(self, jid: Union[str, JID]) -> Union[BareContact,
-                                                              GroupchatContact]:
+    def get_bare_contact(self,
+                         jid: str | JID
+                         ) -> BareContact | GroupchatContact:
         '''This method gives direct access to the contacts dict.
            This is helpful when performance is essential. In difference to
            get_contact() this method does not create contacts nor can it handle
@@ -230,10 +233,9 @@ class Contacts(BaseModule):
 
     def get_contacts_with_domain(self,
                                  domain: str
-                                 ) -> list[Union[BareContact,
-                                                 GroupchatContact]]:
+                                 ) -> list[BareContact | GroupchatContact]:
 
-        contacts: list[Union[BareContact, GroupchatContact]] = []
+        contacts: list[BareContact | GroupchatContact] = []
         for contact in self._contacts.values():
             if contact.jid.domain == domain:
                 contacts.append(contact)
@@ -251,7 +253,7 @@ class Contacts(BaseModule):
             contact.force_chatstate_update()
 
 
-class CommonContact(Observable):
+class CommonContact(Observable, ClientModules):
     def __init__(self,
                  logger: LogAdapter,
                  jid: JID,
@@ -259,6 +261,7 @@ class CommonContact(Observable):
                  ) -> None:
 
         Observable.__init__(self, logger)
+        ClientModules.__init__(self, account)
         self._jid = jid
         self._account = account
 
@@ -272,9 +275,6 @@ class CommonContact(Observable):
         return (self._account == obj.account and
                 obj._jid == self._jid)
 
-    def _module(self, name: str) -> BaseModule:
-        return app.get_client(self._account).get_module(name)
-
     @property
     def jid(self) -> JID:
         return self._jid
@@ -283,15 +283,16 @@ class CommonContact(Observable):
     def account(self) -> str:
         return self._account
 
-    def _on_signal(self,
-                   _contact: Union[BareContact,
-                                   ResourceContact,
-                                   GroupchatContact,
-                                   GroupchatParticipant],
-                   signal_name: str,
-                   *args: Any,
-                   **kwargs: Any
-                   ) -> None:
+    def _on_signal(
+        self,
+        _contact: (BareContact |
+                   ResourceContact |
+                   GroupchatContact |
+                   GroupchatParticipant),
+        signal_name: str,
+        *args: Any,
+        **kwargs: Any
+    ) -> None:
         self.notify(signal_name, *args, **kwargs)
 
     def supports(self, requested_feature: str) -> bool:
@@ -329,7 +330,7 @@ class CommonContact(Observable):
         return self._jid
 
     @property
-    def chatstate(self) -> Optional[Chatstate]:
+    def chatstate(self) -> Chatstate | None:
         return None
 
     @property
@@ -360,7 +361,8 @@ class BareContact(CommonContact):
         self.settings = ContactSettings(account, jid)
         self._resources: dict[str, ResourceContact] = {}
 
-        self._avatar_sha = app.storage.cache.get_contact(jid, 'avatar')
+        self._avatar_sha = app.storage.cache.get_contact(
+            account, jid, 'avatar')
 
     @property
     def is_self(self):
@@ -371,12 +373,14 @@ class BareContact(CommonContact):
         return any(resource.supports(requested_feature)
                    for resource in self.iter_resources())
 
+    @property
     def supports_audio(self) -> bool:
         if (self.supports(Namespace.JINGLE_ICE_UDP) and
                 app.is_installed('FARSTREAM')):
             return self.supports(Namespace.JINGLE_RTP_AUDIO)
         return False
 
+    @property
     def supports_video(self) -> bool:
         if (self.supports(Namespace.JINGLE_ICE_UDP) and
                 app.is_installed('FARSTREAM')):
@@ -420,8 +424,8 @@ class BareContact(CommonContact):
     @property
     def is_available(self) -> bool:
         # pylint: disable=R1729
-        return any([contact.is_available for contact
-                    in self._resources.values()])
+        return any(contact.is_available for contact
+                   in self._resources.values())
 
     @property
     def show(self):
@@ -431,7 +435,7 @@ class BareContact(CommonContact):
         return max(show_values)
 
     @property
-    def chatstate(self) -> Optional[Chatstate]:
+    def chatstate(self) -> Chatstate | None:
         chatstates = {contact.chatstate for contact in self._resources.values()}
         chatstates.discard(None)
         if not chatstates:
@@ -444,11 +448,12 @@ class BareContact(CommonContact):
 
     @property
     def name(self) -> str:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         if item is not None and item.name:
             return item.name
 
-        nickname = app.storage.cache.get_contact(self._jid, 'nickname')
+        nickname = app.storage.cache.get_contact(
+            self._account, self._jid, 'nickname')
         if nickname:
             return nickname
         if self._jid.is_domain:
@@ -458,15 +463,19 @@ class BareContact(CommonContact):
         assert self._jid.localpart is not None
         return self._jid.localpart
 
-    def get_tune(self) -> Optional[TuneData]:
-        return self._module('UserTune').get_contact_tune(self._jid)
+    def get_tune(self) -> TuneData | None:
+        return self.get_module('UserTune').get_contact_tune(self._jid)
 
-    def get_location(self) -> Optional[LocationData]:
-        return self._module('UserLocation').get_contact_location(self._jid)
+    def get_location(self) -> LocationData | None:
+        return self.get_module('UserLocation').get_contact_location(self._jid)
 
     @property
-    def avatar_sha(self) -> Optional[str]:
-        return app.storage.cache.get_contact(self._jid, 'avatar')
+    def avatar_sha(self) -> str | None:
+        return app.storage.cache.get_contact(
+            self._account, self._jid, 'avatar')
+
+    def set_avatar_sha(self, sha: str) -> None:
+        app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
 
     def get_avatar(self,
                    size: int,
@@ -514,13 +523,13 @@ class BareContact(CommonContact):
 
         self._avatar_sha = sha
 
-        app.storage.cache.set_contact(self._jid, 'avatar', sha)
+        app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
         app.app.avatar_storage.invalidate_cache(self._jid)
         self.notify('avatar-update')
 
     @property
     def is_in_roster(self) -> bool:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         return item is not None
 
     @property
@@ -531,22 +540,22 @@ class BareContact(CommonContact):
         return disco_info.is_gateway
 
     @property
-    def ask(self) -> Optional[str]:
-        item = self._module('Roster').get_item(self._jid)
+    def ask(self) -> str | None:
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return None
         return item.ask
 
     @property
-    def subscription(self) -> Optional[str]:
-        item = self._module('Roster').get_item(self._jid)
+    def subscription(self) -> str | None:
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return None
         return item.subscription
 
     @property
     def groups(self) -> set[str]:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return set()
         return item.groups
@@ -557,7 +566,7 @@ class BareContact(CommonContact):
 
     @property
     def is_blocked(self) -> bool:
-        return self._module('Blocking').is_blocked(self._jid)
+        return self.get_module('Blocking').is_blocked(self._jid)
 
     def set_blocked(self) -> None:
         self.update_presence(UNKNOWN_PRESENCE)
@@ -598,7 +607,7 @@ class ResourceContact(CommonContact):
         return self._jid.resource
 
     @property
-    def identity_type(self) -> Optional[str]:
+    def identity_type(self) -> str | None:
         disco_info = app.storage.cache.get_last_disco_info(self._jid)
         if disco_info is None:
             return None
@@ -637,12 +646,12 @@ class ResourceContact(CommonContact):
         return self._presence.priority
 
     @property
-    def idle_time(self) -> Optional[float]:
+    def idle_time(self) -> float | None:
         return self._presence.idle_time
 
     @property
-    def chatstate(self) -> Chatstate:
-        return self._module('Chatstate').get_remote_chatstate(self._jid)
+    def chatstate(self) -> Chatstate | None:
+        return self.get_module('Chatstate').get_remote_chatstate(self._jid)
 
     def update_presence(self,
                         presence_data: PresenceData,
@@ -680,7 +689,7 @@ class GroupchatContact(CommonContact):
         return disco_info.is_irc
 
     @property
-    def muc_context(self) -> Optional[str]:
+    def muc_context(self) -> str | None:
         disco_info = self.get_disco()
         if disco_info is None:
             return None
@@ -742,10 +751,13 @@ class GroupchatContact(CommonContact):
         return get_groupchat_name(client, self._jid)
 
     @property
-    def avatar_sha(self) -> Optional[str]:
-        return app.storage.cache.get_muc(self._jid, 'avatar')
+    def avatar_sha(self) -> str | None:
+        return app.storage.cache.get_muc(self._account, self._jid, 'avatar')
 
-    def get_avatar(self, size: int, scale: int) -> Optional[cairo.ImageSurface]:
+    def set_avatar_sha(self, sha: str) -> None:
+        app.storage.cache.set_muc(self._account, self._jid, 'avatar', sha)
+
+    def get_avatar(self, size: int, scale: int) -> cairo.ImageSurface | None:
         transport_icon = None
         disco_info = self.get_disco()
         if disco_info is not None:
@@ -774,50 +786,50 @@ class GroupchatContact(CommonContact):
         for contact in self._resources.values():
             contact.notify('chatstate-update')
 
-    def get_self(self) -> Optional[GroupchatParticipant]:
+    def get_self(self) -> GroupchatParticipant | None:
         nick = self.nickname
         if nick is None:
             return None
         return self.get_resource(nick)
 
     @property
-    def nickname(self) -> Optional[str]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def nickname(self) -> str | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.nick
 
     @property
-    def occupant_jid(self) -> Optional[JID]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def occupant_jid(self) -> JID | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.occupant_jid
 
     @property
-    def subject(self) -> Optional[MucSubject]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def subject(self) -> MucSubject | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.subject
 
     @property
     def is_joined(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return False
         return muc_data.state.is_joined
 
     @property
     def is_joining(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return False
         return muc_data.state.is_joining
 
     @property
     def is_not_joined(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return True
         return muc_data.state.is_not_joined
@@ -828,9 +840,9 @@ class GroupchatContact(CommonContact):
 
     def get_user_nicknames(self) -> list[str]:
         client = app.get_client(self._account)
-        return client.get_module('MUC').get_joined_users(self._jid)
+        return client.get_module('MUC').get_joined_users(str(self._jid))
 
-    def get_disco(self, max_age: int = 0) -> Optional[DiscoInfo]:
+    def get_disco(self, max_age: int = 0) -> DiscoInfo | None:
         return app.storage.cache.get_last_disco_info(self.jid, max_age=max_age)
 
     def can_notify(self) -> bool:
@@ -879,10 +891,12 @@ class GroupchatParticipant(CommonContact):
 
     @property
     def room(self) -> GroupchatContact:
-        return self._module('Contacts').get_bare_contact(self.jid.bare)
+        contact = self.get_module('Contacts').get_bare_contact(self.jid.bare)
+        assert isinstance(contact, GroupchatContact)
+        return contact
 
     @property
-    def muc_context(self) -> Optional[str]:
+    def muc_context(self) -> str | None:
         return self.room.muc_context
 
     @property
@@ -907,7 +921,7 @@ class GroupchatParticipant(CommonContact):
         return self._presence.status
 
     @property
-    def idle_time(self) -> Optional[float]:
+    def idle_time(self) -> float | None:
         return self._presence.idle_time
 
     @property
@@ -916,15 +930,17 @@ class GroupchatParticipant(CommonContact):
         return self._jid.resource
 
     @property
-    def real_jid(self) -> Optional[JID]:
+    def real_jid(self) -> JID | None:
         return self._presence.real_jid
 
-    def get_real_contact(self) -> Optional[BareContact]:
+    def get_real_contact(self) -> BareContact | None:
         jid = self._presence.real_jid
         if jid is None:
             return None
-        return self._client.get_module('Contacts').get_contact(
+        contact = self._client.get_module('Contacts').get_contact(
             jid.new_as_bare())
+        assert isinstance(contact, BareContact)
+        return contact
 
     @property
     def affiliation(self) -> Affiliation:
@@ -935,12 +951,13 @@ class GroupchatParticipant(CommonContact):
         return self._presence.role
 
     @property
-    def chatstate(self) -> Chatstate:
-        return self._module('Chatstate').get_remote_chatstate(self._jid)
+    def chatstate(self) -> Chatstate | None:
+        return self.get_module('Chatstate').get_remote_chatstate(self._jid)
 
     @property
-    def avatar_sha(self) -> str:
-        return self._client.get_module('VCardAvatars').get_avatar_sha(self._jid)
+    def avatar_sha(self) -> str | None:
+        return self._client.get_module('VCardAvatars').get_avatar_sha(
+            self._jid)
 
     def get_avatar(self,
                    size: int,
@@ -964,10 +981,14 @@ class GroupchatParticipant(CommonContact):
     def type_string(self) -> str:
         return 'pm'
 
+    @property
+    def occupant_id(self) -> str | None:
+        return self._presence.occupant_id
 
-def can_add_to_roster(contact: Union[BareContact,
-                                     GroupchatContact,
-                                     GroupchatParticipant]) -> bool:
+
+def can_add_to_roster(
+    contact: BareContact | GroupchatContact | GroupchatParticipant
+) -> bool:
 
     if isinstance(contact, GroupchatContact):
         return False
