@@ -18,7 +18,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 import time
@@ -187,10 +186,10 @@ class MUC(BaseModule):
                 self._muc_service_jid = info.jid
                 raise nbxmpp.NodeProcessed
 
-    def get_muc_data(self, room_jid: str) -> Optional[MUCData]:
+    def get_muc_data(self, room_jid: JID) -> MUCData | None:
         return self._mucs.get(room_jid)
 
-    def set_password(self, room_jid: str, password: str) -> None:
+    def set_password(self, room_jid: JID, password: str) -> None:
         muc_data = self.get_muc_data(room_jid)
         muc_data.password = password
 
@@ -226,9 +225,9 @@ class MUC(BaseModule):
 
     def _create_muc_data(self,
                          room_jid: str,
-                         nick: Optional[str],
-                         password: Optional[str],
-                         config: Optional[dict[str, Any]]
+                         nick: str | None,
+                         password: str | None,
+                         config: dict[str, Any] | None
                          ) -> MUCData:
         if not nick:
             nick = get_group_chat_nick(self._account, room_jid)
@@ -242,10 +241,10 @@ class MUC(BaseModule):
         return MUCData(room_jid, nick, password, config)
 
     def join(self,
-             jid: str,
-             nick: Optional[str] = None,
-             password: Optional[str] = None,
-             config: Optional[dict[str, Any]] = None
+             jid: JID,
+             nick: str | None = None,
+             password: str | None = None,
+             config: dict[str, Any] | None = None
              ) -> None:
         if not app.account_is_available(self._account):
             return
@@ -352,8 +351,8 @@ class MUC(BaseModule):
         self._con.send_stanza(presence)
 
     def leave(self,
-              room_jid: str,
-              reason: Optional[str] = None
+              room_jid: JID,
+              reason: str | None = None
               ) -> None:
         self._log.info('Leave MUC: %s', room_jid)
 
@@ -379,7 +378,7 @@ class MUC(BaseModule):
         room.set_not_joined()
         room.notify('room-left')
 
-    def abort_join(self, room_jid: str) -> None:
+    def abort_join(self, room_jid: JID) -> None:
         self._remove_rejoin_timeout(room_jid)
 
         muc = self._mucs[room_jid]
@@ -415,7 +414,7 @@ class MUC(BaseModule):
 
     @staticmethod
     def _apply_config(form: SimpleDataForm,
-                      config: Optional[dict[str, Any]] = None
+                      config: dict[str, Any] | None = None
                       ) -> None:
         default_config = get_default_muc_config()
         if config is not None:
@@ -474,7 +473,7 @@ class MUC(BaseModule):
                 status=message,
                 idle_time=idle)
 
-    def change_nick(self, room_jid: str, new_nick: str) -> None:
+    def change_nick(self, room_jid: JID, new_nick: str) -> None:
         status, message, _idle = self._con.get_presence_state()
         self._con.get_module('Presence').send_presence(
             f'{room_jid}/{new_nick}',
@@ -760,15 +759,22 @@ class MUC(BaseModule):
     def _process_user_presence(self,
                                properties: PresenceProperties
                                ) -> MUCPresenceData:
+
         jid = properties.jid
-        muc_presence = MUCPresenceData.from_presence(properties)
+        group_contact = self._client.get_module('Contacts').get_contact(
+            jid.bare, groupchat=True)
+        occupant_id = None
+        if group_contact.supports(Namespace.OCCUPANT_ID):
+            occupant_id = properties.occupant_id
+
+        muc_presence = MUCPresenceData.from_presence(properties, occupant_id)
         if not muc_presence.available:
             self._joined_users[jid.bare].pop(jid.resource)
         else:
             self._joined_users[jid.bare][jid.resource] = muc_presence
         return muc_presence
 
-    def _is_user_joined(self, jid: Optional[JID]) -> bool:
+    def _is_user_joined(self, jid: JID | None) -> bool:
         try:
             self._joined_users[jid.bare][jid.resource]
         except KeyError:
@@ -854,7 +860,7 @@ class MUC(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def cancel_password_request(self, room_jid: str) -> None:
+    def cancel_password_request(self, room_jid: JID) -> None:
         self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
     def _room_join_complete(self, muc_data: MUCData):
@@ -948,7 +954,7 @@ class MUC(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def cancel_captcha(self, room_jid: str) -> None:
+    def cancel_captcha(self, room_jid: JID) -> None:
         muc_data = self._mucs.get(room_jid)
         if muc_data is None:
             return
@@ -961,7 +967,7 @@ class MUC(BaseModule):
         self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
     def send_captcha(self,
-                     room_jid: str,
+                     room_jid: JID,
                      form_node: SimpleDataForm
                      ) -> None:
         self._set_muc_state(room_jid, MUCJoinedState.JOINING)
@@ -1060,7 +1066,7 @@ class MUC(BaseModule):
     def invite(self,
                room: JID,
                jid: JID,
-               reason: Optional[str] = None,
+               reason: str | None = None,
                continue_: bool = False
                ) -> str:
 
