@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import locale
 import logging
@@ -34,7 +33,7 @@ from gajim.common import types
 from gajim.common.const import ClientState
 from gajim.common.i18n import _
 from gajim.common.i18n import p_
-from gajim.common.settings import AllSettingsT
+from gajim.common.setting_values import AllSettingsT
 
 from gajim.gtk.const import Setting
 from gajim.gtk.const import SettingKind
@@ -42,6 +41,7 @@ from gajim.gtk.const import SettingType
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
 from gajim.gtk.menus import build_accounts_menu
+from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
 from gajim.gtk.settings import PopoverSetting
 from gajim.gtk.settings import SettingsBox
 from gajim.gtk.settings import SettingsDialog
@@ -168,7 +168,7 @@ class AccountsWindow(Gtk.ApplicationWindow):
         if not initial:
             self._accounts[account].show()
 
-    def select_account(self, account: str, page: Optional[str] = None) -> None:
+    def select_account(self, account: str, page: str | None = None) -> None:
         try:
             self._accounts[account].select(page)
         except KeyError:
@@ -327,6 +327,7 @@ class AccountSubMenu(Gtk.ListBox):
         self.add(BackMenuItem())
         self.add(PageMenuItem('general', _('General')))
         self.add(PageMenuItem('privacy', _('Privacy')))
+        self.add(PageMenuItem('encryption-omemo', _('Encryption (OMEMO)')))
         self.add(PageMenuItem('connection', _('Connection')))
         self.add(PageMenuItem('advanced', _('Advanced')))
         self.add(RemoveMenuItem())
@@ -429,6 +430,8 @@ class PageMenuItem(MenuItem):
             icon = 'avatar-default-symbolic'
         elif name == 'privacy':
             icon = 'preferences-system-privacy-symbolic'
+        elif name == 'encryption-omemo':
+            icon = 'channel-secure-symbolic'
         elif name == 'connection':
             icon = 'preferences-system-network-symbolic'
         elif name == 'advanced':
@@ -454,14 +457,15 @@ class Account:
         self._settings = settings
 
         self._settings.add_page(GeneralPage(account))
-        self._settings.add_page(ConnectionPage(account))
         self._settings.add_page(PrivacyPage(account))
+        self._settings.add_page(EncryptionOMEMOPage(account))
+        self._settings.add_page(ConnectionPage(account))
         self._settings.add_page(AdvancedPage(account))
 
         self._account_row = AccountRow(account)
         self._menu.add_account(self._account_row)
 
-    def select(self, page_name: Optional[str] = None) -> None:
+    def select(self, page_name: str | None = None) -> None:
         self._account_row.emit('activate')
         if page_name is not None:
             self._menu.set_page(self._account, page_name)
@@ -639,6 +643,7 @@ class GenericSettingPage(Gtk.Box):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=12)
         self.set_valign(Gtk.Align.START)
         self.set_vexpand(True)
+        self.get_style_context().add_class('settings-page')
         self.account = account
 
         self.listbox = SettingsBox(account)
@@ -757,7 +762,7 @@ class PrivacyPage(GenericSettingPage):
 
     def __init__(self, account: str) -> None:
         self._account = account
-        self._client: Optional[types.Client] = None
+        self._client: types.Client | None = None
         if app.account_is_connected(account):
             self._client = app.get_client(account)
 
@@ -778,7 +783,22 @@ class PrivacyPage(GenericSettingPage):
             'disabled': _('Disabled'),
         }
 
+        encryption_entries = {
+            '': _('Unencrypted'),
+            'OMEMO': 'OMEMO',
+            'OpenPGP': 'OpenPGP',
+            'PGP': 'PGP',
+        }
+
         settings = [
+            Setting(SettingKind.POPOVER,
+                    _('Default Encryption'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'encryption_default',
+                    desc=_('Encryption method to use '
+                           'unless overridden on a per-contact basis'),
+                    props={'entries': encryption_entries}),
+
             Setting(SettingKind.SWITCH,
                     _('Idle Time'),
                     SettingType.ACCOUNT_CONFIG,
@@ -915,6 +935,39 @@ class PrivacyPage(GenericSettingPage):
             'send_marker', None, context='private')
 
 
+class EncryptionOMEMOPage(GenericSettingPage):
+
+    name = 'encryption-omemo'
+
+    def __init__(self, account: str) -> None:
+        settings = [
+            Setting(SettingKind.SWITCH,
+                    _('Blind Trust'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'omemo_blind_trust',
+                    desc=_('Blindly trust new devices until you verify them'))
+        ]
+        GenericSettingPage.__init__(self, account, settings)
+
+        heading = Gtk.Label(label=_('Trust Management'))
+        heading.get_style_context().add_class('bold')
+        heading.set_xalign(0)
+        self.add(heading)
+
+        btbv_label = Gtk.Label()
+        btbv_label.set_xalign(0)
+        wiki_url = 'https://dev.gajim.org/gajim/gajim/-/wikis/help/OMEMO'
+        link_text = _('Read more about blind trust')
+        markup = f'<a href="{wiki_url}">{link_text}</a>'
+        btbv_label.set_markup(markup)
+        self.add(btbv_label)
+
+        omemo_trust_manager = OMEMOTrustManager(account)
+        omemo_trust_manager.set_margin_top(18)
+        self.pack_end(omemo_trust_manager, True, True, 0)
+        self.reorder_child(omemo_trust_manager, 0)
+
+
 class ConnectionPage(GenericSettingPage):
 
     name = 'connection'
