@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import locale
 import logging
@@ -36,6 +35,7 @@ from gajim.common.events import MUCNicknameChanged
 from gajim.common.helpers import get_uf_affiliation
 from gajim.common.helpers import get_uf_role
 from gajim.common.helpers import jid_is_blocked
+from gajim.common.i18n import p_
 from gajim.common.modules.contacts import GroupchatContact
 
 from gajim.gtk.builder import get_builder
@@ -84,7 +84,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._tooltip = GCTooltip()
 
         self._ui = get_builder('groupchat_roster.ui')
-        self.add(self._ui.scrolled)
+        self.add(self._ui.box)
 
         self._contact_refs: dict[str, Gtk.TreeRowReference] = {}
         self._group_refs: dict[str, Gtk.TreeRowReference] = {}
@@ -93,7 +93,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._store.set_sort_func(Column.TEXT, self._tree_compare_iters)
 
         self._roster = self._ui.roster_treeview
-        self._roster.set_search_equal_func(self._search_func)
+
+        self._filter_string = ''
+        self._modelfilter = self._store.filter_new()
+        self._modelfilter.set_visible_func(self._visible_func)
         self._roster.set_has_tooltip(True)
 
         self._ui.contact_column.set_fixed_width(
@@ -181,6 +184,11 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._tooltip.clear_tooltip()
             return False
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            self._tooltip.clear_tooltip()
+            return False
+
         iter_ = None
         try:
             iter_ = self._store.get_iter(path)
@@ -201,16 +209,26 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         tooltip.set_custom(widget)
         return value
 
-    @staticmethod
-    def _search_func(model: Gtk.TreeModel,
-                     _column: int,
-                     search_text: str,
-                     iter_: Gtk.TreeIter
-                     ) -> bool:
+    def _on_search_changed(self, widget: Gtk.SearchEntry) -> None:
+        self._filter_string = widget.get_text().lower()
+        self._modelfilter.refilter()
+        self._roster.expand_all()
 
-        return search_text.lower() not in model[iter_][1].lower()
+    def _visible_func(self,
+                      model: Gtk.TreeModelFilter,
+                      iter_: Gtk.TreeIter,
+                      *_data: Any
+                      ) -> bool:
 
-    def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
+        if not self._filter_string:
+            return True
+
+        if not model[iter_][Column.IS_CONTACT]:
+            return True
+
+        return self._filter_string in model[iter_][Column.TEXT].lower()
+
+    def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
             ref = self._group_refs[group_name]
         except KeyError:
@@ -221,7 +239,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             return None
         return self._store.get_iter(path)
 
-    def _get_contact_iter(self, nick: str) -> Optional[Gtk.TreeIter]:
+    def _get_contact_iter(self, nick: str) -> Gtk.TreeIter | None:
         try:
             ref = self._contact_refs[nick]
         except KeyError:
@@ -340,7 +358,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                              renderer: Gtk.CellRenderer,
                              model: Gtk.TreeModel,
                              iter_: Gtk.TreeIter,
-                             _user_data: Optional[object]
+                             _user_data: object | None
                              ) -> None:
 
         has_parent = bool(model.iter_parent(iter_))
@@ -367,7 +385,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                  _column: Gtk.TreeViewColumn
                                  ) -> None:
 
-        iter_ = self._store.get_iter(path)
+        child_path = self._modelfilter.convert_path_to_child_path(path)
+        assert child_path is not None
+
+        iter_ = self._store.get_iter(child_path)
         if self._store.iter_parent(iter_) is None:
             # This is a group row
             return
@@ -411,6 +432,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if path is None:
             return
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            return
+
         iter_ = self._store.get_iter(path)
         if self._store.iter_parent(iter_) is None:
             # Group row
@@ -455,7 +480,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                             model: Gtk.TreeModel,
                             iter1: Gtk.TreeIter,
                             iter2: Gtk.TreeIter,
-                            _user_data: Optional[object]
+                            _user_data: object | None
                             ) -> int:
         '''
         Compare two iterators to sort them
@@ -466,10 +491,15 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             nick1 = model[iter1][Column.NICK_OR_GROUP]
             nick2 = model[iter2][Column.NICK_OR_GROUP]
 
+            assert self._contact is not None
+            our_nick = self._contact.nickname
+            if our_nick in (nick1, nick2):
+                # Always show our nickname at the top
+                return -1 if our_nick == nick1 else 1
+
             if not app.settings.get('sort_by_show_in_muc'):
                 return locale.strcoll(nick1.lower(), nick2.lower())
 
-            assert self._contact is not None
             contact1 = self._contact.get_resource(nick1)
             contact2 = self._contact.get_resource(nick2)
 
@@ -512,7 +542,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._add_contact(participant)
 
         self.enable_sort(True)
-        self._roster.set_model(self._store)
+        self._roster.set_model(self._modelfilter)
+
+        self._ui.search_entry.set_text('')
         self._roster.expand_all()
 
     def _unload_roster(self) -> None:
@@ -550,6 +582,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._draw_avatar(contact)
 
         name = GLib.markup_escape_text(contact.name)
+        self_contact = self._contact.get_self()
+        if self_contact is not None and self_contact.name == nick:
+            name = p_('own nickname in group chat', '%s (You)' % nick)
 
         # Strike name if blocked
         fjid = f'{self._contact.jid}/{nick}'
