@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
@@ -44,7 +43,9 @@ from gajim.common.helpers import play_sound
 from gajim.common.helpers import show_in_folder
 from gajim.common.i18n import _
 from gajim.common.modules.bytestream import is_transfer_active
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.plugins.manifest import PluginManifest
 from gajim.plugins.repository import PluginRepository
 
@@ -441,7 +442,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   _param: Optional[GLib.Variant]) -> Optional[int]:
+                   _param: GLib.Variant | None) -> int | None:
 
         action_name = action.get_name()
         log.info('Activate action: %s', action_name)
@@ -532,6 +533,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         elif action_name == 'preview-save-as':
             def _on_ok(paths: list[str]) -> None:
+                if not paths:
+                    ErrorDialog(
+                        _('Could not save file'),
+                        _('Could not save file to selected directory.'),
+                        transient_for=self)
+                    return
+
                 target = paths[0]
                 assert preview is not None
                 assert preview.orig_path is not None
@@ -755,7 +763,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._account_side_bar.activate_account_page(account)
         self._main_stack.show_account(account)
 
-    def get_active_workspace(self) -> Optional[str]:
+    def get_active_workspace(self) -> str | None:
         return self._workspace_side_bar.get_active_workspace()
 
     def is_chat_active(self, account: str, jid: JID) -> bool:
@@ -801,7 +809,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             self.add_workspace(workspace_id)
 
     def add_workspace(self,
-                      workspace_id: Optional[str] = None,
+                      workspace_id: str | None = None,
                       switch: bool = True) -> str:
 
         if workspace_id is None:
@@ -967,7 +975,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                  jid: JID,
                  type_: str,
                  select: bool = False,
-                 message: Optional[str] = None
+                 message: str | None = None
                  ) -> None:
 
         workspace_id = self._get_suitable_workspace(account, jid)
@@ -993,6 +1001,12 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                                                'pm',
                                                select=select)
 
+    def clear_chat_list_row(self, account: str, jid: JID) -> None:
+        chat_list_stack = self._chat_page.get_chat_list_stack()
+        chat_list = chat_list_stack.find_chat(account, jid)
+        if chat_list is not None:
+            chat_list.clear_chat_list_row(account, jid)
+
     def select_chat(self, account: str, jid: JID) -> None:
         self._app_side_bar.unselect_all()
         self._account_side_bar.unselect_all()
@@ -1026,8 +1040,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def add_app_message(self,
                         category: str,
-                        new_version: Optional[str] = None,
-                        new_setup_url: Optional[str] = None
+                        new_version: str | None = None,
+                        new_setup_url: str | None = None
                         ) -> None:
 
         self._app_page.add_app_message(category, new_version, new_setup_url)
@@ -1097,6 +1111,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         client.get_module('ChatMarkers').send_displayed_marker(
             contact,
             last_message.message_id,
@@ -1120,7 +1136,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def get_preferred_ft_method(self,
                                 contact: types.ChatContactT
-                                ) -> Optional[str]:
+                                ) -> str | None:
 
         ft_pref = app.settings.get_account_setting(
             contact.account,
@@ -1148,21 +1164,22 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def show_add_join_groupchat(self,
                                 account: str,
                                 jid: str,
-                                nickname: Optional[str] = None,
-                                password: Optional[str] = None
+                                nickname: str | None = None,
+                                password: str | None = None
                                 ) -> None:
 
-        if not self.chat_exists(account, JID.from_string(jid)):
+        jid_ = JID.from_string(jid)
+        if not self.chat_exists(account, jid_):
             client = app.get_client(account)
             client.get_module('MUC').join(
-                jid, nick=nickname, password=password)
+                jid_, nick=nickname, password=password)
 
-        self.add_group_chat(account, JID.from_string(jid), select=True)
+        self.add_group_chat(account, jid_, select=True)
 
     def start_chat_from_jid(self,
                             account: str,
                             jid: str,
-                            message: Optional[str] = None
+                            message: str | None = None
                             ) -> None:
 
         jid_ = JID.from_string(jid)
@@ -1191,12 +1208,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         client = app.get_client(account)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         if contact.is_blocked:
             client.get_module('Blocking').unblock([jid])
             return
 
         # TODO: Keep "confirm_block" setting?
-        def _block_contact(report: Optional[str] = None) -> None:
+        def _block_contact(report: str | None = None) -> None:
             client.get_module('Blocking').block([contact.jid], report)
             self._chat_page.remove_chat(account, contact.jid)
 
@@ -1223,6 +1241,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             client.get_module('Roster').delete_item(jid)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         sec_text = _('You are about to remove %(name)s (%(jid)s) from '
                      'your contact list.\n') % {
                          'name': contact.name,
@@ -1308,7 +1328,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.settings.set('mainwin_height', window_height)
         app.settings.save()
 
-        def on_continue2(message: Optional[str]) -> None:
+        def on_continue2(message: str | None) -> None:
             if 'file_transfers' not in app.interface.instances:
                 app.app.start_shutdown(message=message)
                 return
@@ -1336,7 +1356,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                 return
             app.app.start_shutdown(message=message)
 
-        def on_continue(message: Optional[str]) -> None:
+        def on_continue(message: str | None) -> None:
             if message is None:
                 # user pressed Cancel to change status message dialog
                 return
