@@ -17,12 +17,11 @@
 
 from __future__ import annotations
 
-from typing import Callable
 from typing import cast
-from typing import Iterator
-from typing import Optional
 
 import textwrap
+from collections.abc import Callable
+from collections.abc import Iterator
 from datetime import datetime
 from urllib.parse import quote
 
@@ -43,8 +42,10 @@ from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import _
 from gajim.common.i18n import get_short_lang_code
 from gajim.common.i18n import p_
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import can_add_to_roster
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.preview import Preview
 from gajim.common.structs import URI
 from gajim.common.util.text import escape_iri_path_segment
@@ -75,7 +76,6 @@ def get_self_contact_menu(contact: types.BareContact) -> GajimMenu:
 
     params = RemoveHistoryActionParams(account=account, jid=jid)
     menu.add_item(_('Remove History…'), 'app.remove-history', params)
-    menu.add_item(_('Search…'), 'win.search-history', None)
     return menu
 
 
@@ -83,7 +83,6 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
     account = contact.account
 
     menu = GajimMenu()
-    menu.add_item(_('Details'), 'win.show-contact-info')
     menu.add_item(_('Block Contact…'),
                   f'app.{account}-block-contact',
                   str(contact.jid))
@@ -94,8 +93,6 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
     menu.add_item(_('Start Voice Call…'), 'win.start-voice-call')
     menu.add_item(_('Start Video Call…'), 'win.start-video-call')
 
-    menu.add_item(_('Search…'), 'win.search-history', None)
-
     if can_add_to_roster(contact):
         params = AccountJidParam(account=account, jid=contact.jid)
         menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
@@ -108,9 +105,7 @@ def get_private_chat_menu(contact: types.GroupchatParticipant) -> GajimMenu:
 
     value = GLib.Variant('as', [''])
 
-    menu.add_item(_('Details'), 'win.show-contact-info')
     menu.add_item(_('Upload File…'), 'win.send-file-httpupload', value)
-    menu.add_item(_('Search…'), 'win.search-history')
 
     if can_add_to_roster(contact):
         params = AccountJidParam(account=contact.account, jid=contact.jid)
@@ -131,11 +126,9 @@ def get_send_file_submenu() -> GajimMenu:
 
 def get_groupchat_menu(contact: GroupchatContact) -> GajimMenu:
     menuitems: MenuItemListT = [
-        (_('Details'), 'win.show-contact-info', None),
         (_('Change Nickname…'), 'win.muc-change-nickname', None),
         (_('Request Voice'), 'win.muc-request-voice', None),
         (_('Execute Command…'), 'win.muc-execute-command', ''),
-        (_('Search…'), 'win.search-history', None)
     ]
 
     return GajimMenu.from_list(menuitems)
@@ -207,7 +200,7 @@ def build_accounts_menu() -> None:
                                             'app.add-contact::')
         acc_menu.insert_item(1, add_contact_item)
         menubar.remove(menu_position)
-        menubar.insert_submenu(menu_position, _('Accounts'), acc_menu)
+        menubar.insert_submenu(menu_position, _('_Accounts'), acc_menu)
 
 
 def get_encryption_menu() -> GajimMenu:
@@ -502,6 +495,8 @@ def get_chat_list_row_menu(workspace_id: str,
 
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
+    assert isinstance(
+        contact, BareContact | GroupchatContact | GroupchatParticipant)
 
     menu = GajimMenu()
 
@@ -663,7 +658,7 @@ def get_groupchat_participant_menu(account: str,
     return menu
 
 
-def get_component_search_menu(jid: Optional[str], copy_text: str) -> GajimMenu:
+def get_component_search_menu(jid: str | None, copy_text: str) -> GajimMenu:
     menuitems: MenuItemListT = [
         (_('Copy'), 'app.copy-text', copy_text),
     ]
@@ -680,16 +675,22 @@ def get_chat_row_menu(contact: types.ChatContactT,
                       name: str,
                       text: str,
                       timestamp: datetime,
-                      message_id: Optional[str],
-                      stanza_id: Optional[str],
-                      log_line_id: Optional[int]
+                      message_id: str | None,
+                      stanza_id: str | None,
+                      log_line_id: int | None
                       ) -> GajimMenu:
 
     menu_items: MenuItemListT = []
 
     format_string = app.settings.get('date_time_format')
     timestamp_formatted = timestamp.strftime(format_string)
-    copy_text = f'{timestamp_formatted} - {name}: {text}'
+
+    copy_text = f'{timestamp_formatted} - {name}: '
+    if text.startswith(('```', '> ')):
+        # Prepend a line break in order to keep code block/quotes rendering
+        copy_text += '\n'
+    copy_text += text
+
     menu_items.append(
         (p_('Message row action', 'Copy'),
          'win.copy-message',
@@ -732,8 +733,7 @@ def get_chat_row_menu(contact: types.ChatContactT,
 
         if disco_info.has_message_moderation and is_allowed:
             show_retract = True
-    if show_retract:
-        assert stanza_id is not None
+    if show_retract and stanza_id is not None:
         param = RetractMessageParam(
             account=contact.account,
             jid=contact.jid,
@@ -809,7 +809,7 @@ def get_workspace_menu(workspace_id: str) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
-def escape_mnemonic(label: Optional[str]) -> Optional[str]:
+def escape_mnemonic(label: str | None) -> str | None:
     if label is None:
         return None
     # Underscore inside a label means the next letter is a keyboard
