@@ -15,10 +15,9 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import Optional
-from typing import Union
+
+from collections.abc import Callable
 
 from gi.repository import Gio
 from gi.repository import GLib
@@ -35,6 +34,9 @@ from gajim.common.events import SubscribePresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
 from gajim.common.helpers import get_groupchat_name
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
 from gajim.gtk.menus import get_subscription_menu
 from gajim.gtk.util import open_window
@@ -42,7 +44,7 @@ from gajim.gtk.util import open_window
 NotificationActionListT = list[
     tuple[str,
           Callable[[Gio.SimpleAction, GLib.Variant], Any],
-          Optional[str]]]
+          str | None]]
 
 
 class NotificationManager(Gtk.ListBox):
@@ -137,6 +139,7 @@ class NotificationManager(Gtk.ListBox):
         self._client.get_module('Presence').subscribed(jid)
         jid = JID.from_string(jid)
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         if not contact.is_in_roster:
             open_window('AddContact', account=self._account,
                         jid=jid, nick=nickname or contact.name)
@@ -189,7 +192,7 @@ class NotificationManager(Gtk.ListBox):
     def _deny_request(self, jid: str) -> None:
         self._client.get_module('Presence').unsubscribed(jid)
 
-    def _get_notification_row(self, jid: str) -> Optional[NotificationRow]:
+    def _get_notification_row(self, jid: str) -> NotificationRow | None:
         rows = cast(list[NotificationRow], self.get_children())
         for row in rows:
             if row.jid == jid:
@@ -213,6 +216,7 @@ class NotificationManager(Gtk.ListBox):
             if not nick:
                 contact = self._client.get_module('Contacts').get_contact(
                     event.jid)
+                assert isinstance(contact, BareContact)
                 nick = contact.name
             text = _('%s asks you to share your status') % nick
 
@@ -238,6 +242,7 @@ class NotificationManager(Gtk.ListBox):
 
             contact = self._client.get_module('Contacts').get_contact(
                 event.jid)
+            assert isinstance(contact, BareContact)
             text = _('%s stopped sharing their status') % contact.name
 
             app.ged.raise_event(
@@ -261,10 +266,12 @@ class NotificationManager(Gtk.ListBox):
 
         jid = event.from_.bare
         client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.muc)
-        if (contact.muc_context == 'private' and
+        muc_contact = client.get_module('Contacts').get_contact(event.muc)
+        assert isinstance(muc_contact, GroupchatContact)
+        if (muc_contact.muc_context == 'private' and
                 not event.muc.bare_match(event.from_)):
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             text = _('%(contact)s invited you to %(chat)s') % {
                 'contact': contact.name,
                 'chat': event.info.muc_name}
@@ -310,10 +317,16 @@ class NotificationRow(Gtk.ListBoxRow):
         label.set_max_width_chars(30)
         return label
 
-    def _generate_avatar_image(self, jid: Union[str, JID]) -> Gtk.Image:
+    def _generate_avatar_image(self, jid: str | JID) -> Gtk.Image:
         contact = self._client.get_module('Contacts').get_contact(jid)
-        surface = contact.get_avatar(
-            AvatarSize.ROSTER, self.get_scale_factor(), add_show=False)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+        if isinstance(contact, GroupchatContact):
+            surface = contact.get_avatar(
+                AvatarSize.ROSTER, self.get_scale_factor())
+        else:
+            surface = contact.get_avatar(
+                AvatarSize.ROSTER, self.get_scale_factor(), add_show=False)
         image = Gtk.Image.new_from_surface(surface)
         image.set_valign(Gtk.Align.CENTER)
         return image
@@ -324,7 +337,7 @@ class SubscriptionRequestRow(NotificationRow):
                  account: str,
                  jid: str,
                  text: str,
-                 user_nick: Optional[str] = None
+                 user_nick: str | None = None
                  ) -> None:
         NotificationRow.__init__(self, account, jid)
         self.type = 'subscribe'
@@ -382,6 +395,7 @@ class UnsubscribedRow(NotificationRow):
         self.grid.attach(image, 1, 1, 1, 2)
 
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         nick_markup = f'<b>{contact.name}</b>'
         nick_label = self._generate_label()
         nick_label.set_tooltip_markup(nick_markup)
@@ -433,10 +447,12 @@ class InvitationReceivedRow(NotificationRow):
         self.grid.attach(title_label, 2, 1, 1, 1)
 
         client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.muc)
-        if (contact.muc_context == 'private' and
+        muc_contact = client.get_module('Contacts').get_contact(event.muc)
+        assert isinstance(muc_contact, GroupchatContact)
+        if (muc_contact.muc_context == 'private' and
                 not event.muc.bare_match(event.from_)):
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             invitation_text = _('%(contact)s invited you to %(chat)s') % {
                 'contact': contact.name,
                 'chat': event.info.muc_name}
@@ -492,6 +508,7 @@ class InvitationDeclinedRow(NotificationRow):
         self.grid.attach(title_label, 2, 1, 1, 1)
 
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         muc_name = get_groupchat_name(self._client, event.muc)
         invitation_text = _('%(contact)s declined your invitation '
                             'to %(chat)s') % {
