@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import Literal
-from typing import Optional
 
 import locale
 import logging
@@ -33,7 +32,6 @@ from nbxmpp.namespaces import Namespace
 
 from gajim.common import app
 from gajim.common import ged
-from gajim.common import types
 from gajim.common.const import AvatarSize
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import StyleAttr
@@ -42,6 +40,7 @@ from gajim.common.events import RosterPush
 from gajim.common.events import RosterReceived
 from gajim.common.helpers import event_filter
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.dialogs import ConfirmationDialog
@@ -152,8 +151,10 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             assert action is not None
             action.connect('activate', func)
 
-    def _get_contact(self, jid: str) -> types.BareContact:
-        return self._client.get_module('Contacts').get_contact(jid)
+    def _get_contact(self, jid: str) -> BareContact:
+        contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
+        return contact
 
     def _on_theme_update(self, _event: ApplicationEvent) -> None:
         self.redraw()
@@ -256,6 +257,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         contact = self._contacts.get_bare_contact(
             model[iter_][Column.JID_OR_GROUP])
+        assert isinstance(contact, BareContact)
         value, widget = self._roster_tooltip.get_tooltip(path, contact)
         tooltip.set_custom(widget)
         return value
@@ -295,6 +297,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         jid = JID.from_string(param.get_string())
         selected_contact = self._contacts.get_contact(jid)
+        assert isinstance(selected_contact, BareContact)
         if selected_contact.is_gateway:
             # Check for transport users in roster and warn about removing the
             # transport if there are any
@@ -306,7 +309,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
             def _on_remove():
                 self._client.get_module('Gateway').unsubscribe(
-                    selected_contact.jid)
+                    str(selected_contact.jid))
 
             if has_transport_contacts:
                 ConfirmationDialog(
@@ -381,6 +384,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                            event: Gdk.EventButton) -> None:
 
         contact = self._contacts.get_bare_contact(jid)
+        assert isinstance(contact, BareContact)
         gateway_register = contact.is_gateway and contact.supports(
             Namespace.REGISTER)
 
@@ -406,7 +410,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         return self._filter_string in model[iter_][Column.TEXT].lower()
 
-    def _get_contact_visible(self, contact: types.BareContact) -> bool:
+    def _get_contact_visible(self, contact: BareContact) -> bool:
         if self._filter_enabled:
             return self._filter_string in contact.name.lower()
 
@@ -451,14 +455,14 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._set_model()
         self._roster.expand_all()
 
-    def _connect_contact_signals(self, contact: types.BareContact) -> None:
+    def _connect_contact_signals(self, contact: BareContact) -> None:
         contact.connect('presence-update', self._on_contact_update)
         contact.connect('caps-update', self._on_contact_update)
         contact.connect('avatar-update', self._on_contact_update)
         contact.connect('blocking-update', self._on_contact_update)
 
     def _on_contact_update(self,
-                           contact: types.BareContact,
+                           contact: BareContact,
                            _signal_name: str) -> None:
 
         self._draw_contact(contact)
@@ -470,6 +474,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
     @event_filter(['account'])
     def _on_roster_push(self, event: RosterPush) -> None:
         contact = self._contacts.get_contact(str(event.item.jid))
+        assert isinstance(contact, BareContact)
 
         if event.item.subscription == 'remove':
             contact.disconnect(self)
@@ -501,7 +506,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         return group_iter
 
     def _add_contact_to_group(self,
-                              contact: types.BareContact,
+                              contact: BareContact,
                               group: str) -> None:
 
         group_iter = self._add_group(group)
@@ -513,13 +518,13 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._contact_refs[contact.jid].append(ref)
 
     def _add_contact_to_groups(self,
-                               contact: types.BareContact,
+                               contact: BareContact,
                                groups: set[str]) -> None:
         for group in groups:
             self._add_contact_to_group(contact, group)
 
     def _remove_contact_from_groups(self,
-                                    contact: types.BareContact,
+                                    contact: BareContact,
                                     groups: set[str]) -> None:
         if not groups:
             return
@@ -539,7 +544,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         self._check_for_empty_groups()
 
-    def _remove_contact(self, contact: types.BareContact) -> None:
+    def _remove_contact(self, contact: BareContact) -> None:
         refs = self._contact_refs.pop(contact.jid)
         for ref in refs:
             iter_ = self._get_iter_from_ref(ref)
@@ -556,7 +561,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             self._store.remove(group_iter)
             del self._group_refs[group]
 
-    def _add_or_update_contact(self, contact: types.BareContact) -> None:
+    def _add_or_update_contact(self, contact: BareContact) -> None:
         new_groups = set(contact.groups or [DEFAULT_GROUP])
         groups = self._get_current_groups(contact.jid)
 
@@ -599,6 +604,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             for child in group.iterchildren():
                 contact = self._contacts.get_bare_contact(
                     child[Column.JID_OR_GROUP])
+                assert isinstance(contact, BareContact)
                 is_visible = self._get_contact_visible(contact)
                 child[Column.VISIBLE] = is_visible
                 if is_visible:
@@ -607,14 +613,14 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             group[Column.VISIBLE] = group_is_visible
         self._roster.expand_all()
 
-    def _draw_contact(self, contact: types.BareContact) -> None:
+    def _draw_contact(self, contact: BareContact) -> None:
         for ref in self._contact_refs[contact.jid]:
             self._draw_contact_row(ref, contact)
         self._roster.expand_all()
 
     def _draw_contact_row(self,
                           ref: Gtk.TreeRowReference,
-                          contact: types.BareContact):
+                          contact: BareContact):
 
         iter_ = self._get_iter_from_ref(ref)
 
@@ -639,7 +645,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         assert path is not None
         return self._store.get_iter(path)
 
-    def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
+    def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
             ref = self._group_refs[group_name]
         except KeyError:
@@ -694,8 +700,10 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
             contact1 = self._contacts.get_bare_contact(
                 model[iter1][Column.JID_OR_GROUP])
+            assert isinstance(contact1, BareContact)
             contact2 = self._contacts.get_bare_contact(
                 model[iter2][Column.JID_OR_GROUP])
+            assert isinstance(contact2, BareContact)
 
             if contact1.show != contact2.show:
                 if contact1.show == PresenceShow.DND:
