@@ -13,14 +13,13 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Optional
-from typing import Union
 
 import time
 
 import nbxmpp
 from gi.repository import Gdk
 from gi.repository import GLib
+from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import GtkSource
 
@@ -32,6 +31,7 @@ from gajim.common.events import AccountEnabled
 from gajim.common.events import StanzaReceived
 from gajim.common.events import StanzaSent
 from gajim.common.i18n import _
+from gajim.common.logging_helpers import get_log_console_handler
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.const import Setting
@@ -41,7 +41,6 @@ from gajim.gtk.dialogs import ErrorDialog
 from gajim.gtk.settings import SettingsDialog
 from gajim.gtk.util import at_the_end
 from gajim.gtk.util import EventHelper
-from gajim.gtk.util import get_source_view_style_scheme
 from gajim.gtk.util import MaxWidthComboBoxText
 from gajim.gtk.util import scroll_to_end
 
@@ -58,21 +57,23 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         self.set_name('XMLConsoleWindow')
 
         self._selected_account = 'AllAccounts'
-        self._selected_send_account: Optional[str] = None
-        self.presence = True
-        self.message = True
-        self.iq = True
-        self.stream = True
-        self.incoming = True
-        self.outgoing = True
-        self.filter_dialog = None
-        self.last_stanza = None
-        self.last_search = ''
+        self._selected_send_account: str | None = None
+        self._filter_dialog: SettingsDialog | None = None
+        self._sent_stanzas = SentSzanzas()
+        self._last_selected_ts = 0
+        self._last_search: str = ''
+
+        self._presence = True
+        self._message = True
+        self._iq = True
+        self._stream = True
+        self._incoming = True
+        self._outgoing = True
 
         self._ui = get_builder('xml_console.ui')
         self.set_titlebar(self._ui.headerbar)
         self._set_titlebar()
-        self.add(self._ui.box)
+        self.add(self._ui.stack)
 
         self._ui.paned.set_position(
             self._ui.paned.get_property('max-position'))
@@ -84,21 +85,36 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         self._combo.set_no_show_all(True)
         self._combo.set_visible(False)
         self._combo.connect('changed', self._on_value_change)
-        for account, label in self._get_accounts():
+        available_accounts = self._get_accounts()
+        for account, label in available_accounts:
             self._combo.append(account, label)
-        self._ui.actionbar.pack_end(self._combo)
+        if available_accounts:
+            self._combo.set_active(0)
+        self._ui.actionbox.pack_end(self._combo, False, False, 0)
+        self._ui.actionbox.reorder_child(self._combo, 1)
 
         self._create_tags()
 
         source_manager = GtkSource.LanguageManager.get_default()
         lang = source_manager.get_language('xml')
-        self._ui.sourceview.get_buffer().set_language(lang)
+        self._ui.protocol_view.get_buffer().set_language(lang)
         self._ui.input_entry.get_buffer().set_language(lang)
 
-        style_scheme = get_source_view_style_scheme()
+        style_scheme_manager = GtkSource.StyleSchemeManager.get_default()
+        style_scheme = style_scheme_manager.get_scheme('solarized-dark')
         if style_scheme is not None:
-            self._ui.sourceview.get_buffer().set_style_scheme(style_scheme)
+            self._ui.protocol_view.get_buffer().set_style_scheme(style_scheme)
             self._ui.input_entry.get_buffer().set_style_scheme(style_scheme)
+            self._ui.log_view.get_buffer().set_style_scheme(style_scheme)
+
+        for record in app.logging_records:
+            self._add_log_record(record)
+
+        log_handler = get_log_console_handler()
+        log_handler.set_callback(self._add_log_record)
+
+        self._ui.stack.connect('notify::visible-child-name',
+                               self._on_stack_child_changed)
 
         self.show_all()
 
@@ -109,21 +125,15 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         self.register_events([
             ('stanza-received', ged.GUI1, self._on_stanza_received),
             ('stanza-sent', ged.GUI1, self._on_stanza_sent),
-            ('style-changed', ged.GUI1, self._on_style_changed),
             ('account-enabled', ged.GUI1, self._on_account_changed),
             ('account-disabled', ged.GUI1, self._on_account_changed)
         ])
 
     def _on_destroy(self, *args: Any) -> None:
+        get_log_console_handler().set_callback(None)
         self._ui.popover.destroy()
         app.check_finalize(self)
 
-    def _on_style_changed(self, *args: Any) -> None:
-        style_scheme = get_source_view_style_scheme()
-        if style_scheme is not None:
-            self._ui.sourceview.get_buffer().set_style_scheme(style_scheme)
-            self._ui.input_entry.get_buffer().set_style_scheme(style_scheme)
-
     def _on_value_change(self, combo: Gtk.ComboBox) -> None:
         self._selected_send_account = combo.get_active_id()
 
@@ -137,9 +147,9 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         self._ui.headerbar.set_subtitle(title)
 
     def _on_account_changed(self,
-                            event: Union[AccountEnabled, AccountDisabled]
+                            event: AccountEnabled | AccountDisabled
                             ) -> None:
-        buf = self._ui.sourceview.get_buffer()
+        buf = self._ui.protocol_view.get_buffer()
 
         if isinstance(event, AccountEnabled):
             buf.create_tag(event.account)
@@ -147,6 +157,14 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             start, end = buf.get_bounds()
             buf.remove_tag_by_name(event.account, start, end)
 
+    def _on_stack_child_changed(self,
+                                _widget: Gtk.Stack,
+                                _pspec: GObject.ParamSpec
+                                ) -> None:
+
+        name = self._ui.stack.get_visible_child_name()
+        self._ui.search_toggle.set_sensitive(name == 'protocol')
+
     def _create_tags(self) -> None:
         tags = [
             'incoming',
@@ -163,7 +181,12 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         tags.append('AccountWizard')
 
         for tag_name in tags:
-            self._ui.sourceview.get_buffer().create_tag(tag_name)
+            self._ui.protocol_view.get_buffer().create_tag(tag_name)
+
+    def _add_log_record(self, message: str) -> None:
+        buf = self._ui.log_view.get_buffer()
+        end_iter = buf.get_end_iter()
+        buf.insert(end_iter, message)
 
     def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
@@ -177,7 +200,10 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             self._on_send()
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_Up):
-            self._on_paste_last()
+            self._on_paste_previous()
+        if (event.state & Gdk.ModifierType.CONTROL_MASK and
+                event.keyval == Gdk.KEY_Down):
+            self._on_paste_next()
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_f):
             self._ui.search_toggle.set_active(
@@ -193,7 +219,6 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         assert isinstance(child, Gtk.Label)
         text = child.get_text()
 
-        # pylint: disable=line-too-long
         input_text = None
         if text == 'Presence':
             input_text = (
@@ -217,7 +242,6 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
                 '<iq to="" type="get" xmlns="jabber:client">\n'
                 '<query xmlns="http://jabber.org/protocol/disco#info">'
                 '</query>\n</iq>')
-        # pylint: enable=line-too-long
 
         if input_text is not None:
             buffer_ = self._ui.input_entry.get_buffer()
@@ -254,13 +278,17 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             client = app.get_client(self._selected_send_account)
             assert isinstance(node, nbxmpp.Protocol)
             client.connection.send_stanza(node)
-            self.last_stanza = stanza
+            self._sent_stanzas.add(stanza)
             buffer_.set_text('')
 
-    def _on_paste_last(self, *args: Any) -> None:
+    def _on_paste_previous(self, *args: Any) -> None:
         buffer_ = self._ui.input_entry.get_buffer()
-        if self.last_stanza is not None:
-            buffer_.set_text(self.last_stanza)
+        buffer_.set_text(self._sent_stanzas.get_previous())
+        self._ui.input_entry.grab_focus()
+
+    def _on_paste_next(self, *args: Any) -> None:
+        buffer_ = self._ui.input_entry.get_buffer()
+        buffer_.set_text(self._sent_stanzas.get_next())
         self._ui.input_entry.grab_focus()
 
     def _on_input(self, button: Gtk.ToggleButton) -> None:
@@ -270,6 +298,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             child2.show()
             self._ui.send.show()
             self._ui.paste.show()
+            self._ui.account_label.show()
             self._combo.show()
             self._ui.menubutton.show()
             self._ui.input_entry.grab_focus()
@@ -277,6 +306,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             child2.hide()
             self._ui.send.hide()
             self._ui.paste.hide()
+            self._ui.account_label.hide()
             self._combo.hide()
             self._ui.menubutton.hide()
 
@@ -296,7 +326,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _find(self, direction: Direction) -> None:
         search_str = self._ui.search_entry.get_text()
-        textbuffer = self._ui.sourceview.get_buffer()
+        textbuffer = self._ui.protocol_view.get_buffer()
         cursor_mark = textbuffer.get_insert()
         current_pos = textbuffer.get_iter_at_mark(cursor_mark)
 
@@ -307,7 +337,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         if last_pos_mark is not None:
             current_pos = textbuffer.get_iter_at_mark(last_pos_mark)
 
-        if search_str != self.last_search:
+        if search_str != self._last_search:
             current_pos = textbuffer.get_start_iter()
 
         if direction == Direction.NEXT:
@@ -328,13 +358,13 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
             match_start, match_end = match
             textbuffer.select_range(match_start, match_end)
             mark = textbuffer.create_mark('last_pos', match_end, True)
-            self._ui.sourceview.scroll_to_mark(mark, 0, True, 0.5, 0.5)
-        self.last_search = search_str
+            self._ui.protocol_view.scroll_to_mark(mark, 0, True, 0.5, 0.5)
+        self._last_search = search_str
 
     @staticmethod
-    def _get_accounts() -> list[tuple[Optional[str], str]]:
+    def _get_accounts() -> list[tuple[str | None, str]]:
         accounts = app.get_accounts_sorted()
-        combo_accounts: list[tuple[Optional[str], str]] = []
+        combo_accounts: list[tuple[str | None, str]] = []
         for account in accounts:
             label = app.get_account_label(account)
             combo_accounts.append((account, label))
@@ -342,54 +372,76 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         return combo_accounts
 
     def _on_filter_options(self, _button: Gtk.Button) -> None:
-        if self.filter_dialog is not None:
-            self.filter_dialog.present()
+        if self._filter_dialog is not None:
+            self._filter_dialog.present()
             return
 
         combo_accounts = self._get_accounts()
         combo_accounts.insert(0, ('AllAccounts', _('All Accounts')))
 
         settings = [
-            Setting(SettingKind.COMBO, _('Account'),
-                    SettingType.VALUE, self._selected_account,
+            Setting(SettingKind.COMBO,
+                    _('Account'),
+                    SettingType.VALUE,
+                    self._selected_account,
                     callback=self._set_account,
                     props={'combo_items': combo_accounts}),
 
-            Setting(SettingKind.SWITCH, 'Presence',
-                    SettingType.VALUE, self.presence,
-                    callback=self._on_setting, data='presence'),
+            Setting(SettingKind.SWITCH,
+                    'Presence',
+                    SettingType.VALUE,
+                    self._presence,
+                    callback=self._on_setting,
+                    data='presence'),
 
-            Setting(SettingKind.SWITCH, 'Message',
-                    SettingType.VALUE, self.message,
-                    callback=self._on_setting, data='message'),
+            Setting(SettingKind.SWITCH,
+                    'Message',
+                    SettingType.VALUE,
+                    self._message,
+                    callback=self._on_setting,
+                    data='message'),
 
-            Setting(SettingKind.SWITCH, 'IQ', SettingType.VALUE, self.iq,
-                    callback=self._on_setting, data='iq'),
+            Setting(SettingKind.SWITCH,
+                    'IQ', SettingType.VALUE,
+                    self._iq,
+                    callback=self._on_setting,
+                    data='iq'),
 
-            Setting(SettingKind.SWITCH, 'Stream Management',
-                    SettingType.VALUE, self.stream,
-                    callback=self._on_setting, data='stream'),
+            Setting(SettingKind.SWITCH,
+                    'Stream Management',
+                    SettingType.VALUE,
+                    self._stream,
+                    callback=self._on_setting,
+                    data='stream'),
 
-            Setting(SettingKind.SWITCH, 'In', SettingType.VALUE, self.incoming,
-                    callback=self._on_setting, data='incoming'),
+            Setting(SettingKind.SWITCH,
+                    'In',
+                    SettingType.VALUE,
+                    self._incoming,
+                    callback=self._on_setting,
+                    data='incoming'),
 
-            Setting(SettingKind.SWITCH, 'Out', SettingType.VALUE, self.outgoing,
-                    callback=self._on_setting, data='outgoing'),
+            Setting(SettingKind.SWITCH,
+                    'Out',
+                    SettingType.VALUE,
+                    self._outgoing,
+                    callback=self._on_setting,
+                    data='outgoing'),
         ]
 
-        self.filter_dialog = SettingsDialog(
+        self._filter_dialog = SettingsDialog(
             self,
             _('Filter'),
             Gtk.DialogFlags.DESTROY_WITH_PARENT,
             settings,
             self._selected_account or 'AllAccounts')
-        self.filter_dialog.connect('destroy', self._on_filter_destroyed)
+        self._filter_dialog.connect('destroy', self._on_filter_destroyed)
 
     def _on_filter_destroyed(self, _widget: Gtk.Widget) -> None:
-        self.filter_dialog = None
+        self._filter_dialog = None
 
     def _on_clear(self, _button: Gtk.Button) -> None:
-        self._ui.sourceview.get_buffer().set_text('')
+        self._ui.protocol_view.get_buffer().set_text('')
 
     def _set_account(self, value: str, _data: Any) -> None:
         self._selected_account = value
@@ -398,7 +450,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         active_accounts = app.settings.get_active_accounts()
         active_accounts.append('AccountWizard')
 
-        table = self._ui.sourceview.get_buffer().get_tag_table()
+        table = self._ui.protocol_view.get_buffer().get_tag_table()
 
         if value == 'AllAccounts':
             for account in active_accounts:
@@ -413,9 +465,9 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
                 tag.set_property('invisible', account != value)
 
     def _on_setting(self, value: bool, data: str) -> None:
-        setattr(self, data, value)
+        setattr(self, f'_{data}', value)
         value = not value
-        table = self._ui.sourceview.get_buffer().get_tag_table()
+        table = self._ui.protocol_view.get_buffer().get_tag_table()
         tag = table.lookup(data)
         if tag is None:
             return
@@ -435,7 +487,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         self._print_stanza(event, 'outgoing')
 
     def _print_stanza(self,
-                      event: Union[StanzaReceived, StanzaSent],
+                      event: StanzaReceived | StanzaSent,
                       kind: str
                       ) -> None:
         if event.account == 'AccountWizard':
@@ -453,7 +505,7 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
 
         is_at_the_end = at_the_end(self._ui.scrolled)
 
-        buffer_ = self._ui.sourceview.get_buffer()
+        buffer_ = self._ui.protocol_view.get_buffer()
         end_iter = buffer_.get_end_iter()
 
         type_ = kind
@@ -466,13 +518,49 @@ class XMLConsoleWindow(Gtk.ApplicationWindow, EventHelper):
         elif stanza.startswith('<r') or stanza.startswith('<a'):
             type_ = 'stream'
 
-        stanza = '<!-- {kind} {time} ({account}) -->\n{stanza}\n\n'.format(
+        text = '<!-- {kind} {time} ({account}) -->\n{stanza}\n\n'.format(
             kind=kind.capitalize(),
             time=time.strftime('%c'),
             account=account_label,
             stanza=stanza)
         buffer_.insert_with_tags_by_name(
-            end_iter, stanza, type_, kind, event.account)
+            end_iter, text, type_, kind, event.account)
 
         if is_at_the_end:
             GLib.idle_add(scroll_to_end, self._ui.scrolled)
+
+
+class SentSzanzas:
+    def __init__(self) -> None:
+        self._sent_stanzas: dict[float, str] = {}
+        self._last_selected_ts = 0
+
+    def add(self, stanza: str) -> None:
+        self._sent_stanzas[time.time()] = stanza
+        self._last_selected_ts = 0
+
+    def get_previous(self) -> str:
+        return self._get(Direction.PREV)
+
+    def get_next(self) -> str:
+        return self._get(Direction.NEXT)
+
+    def _get(self, direction: Direction) -> str:
+        if not self._sent_stanzas:
+            return ''
+
+        if direction == Direction.PREV:
+            for timestamp, stanza in reversed(self._sent_stanzas.items()):
+                if timestamp >= self._last_selected_ts:
+                    continue
+                self._last_selected_ts = timestamp
+                return stanza
+        else:
+            for timestamp, stanza in self._sent_stanzas.items():
+                if timestamp <= self._last_selected_ts:
+                    continue
+                self._last_selected_ts = timestamp
+                return stanza
+
+        self._last_selected_ts = list(self._sent_stanzas.keys())[-1]
+        return self._sent_stanzas[self._last_selected_ts]
